/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import AppKit
import BluetoothControl

class TransportViewController: NSViewController, IncomingRecordControlToUIDelegate {

	// IBOutlets
    @IBOutlet weak var recordButton: NSButton!
    @IBOutlet weak var recordingErrorLabel: NSTextField!
    @IBOutlet var timecodeLabel: NSTextField!
    @IBOutlet var disk1RemainingTimeLabel: NSTextField!
    @IBOutlet var disk2RemainingTimeLabel: NSTextField!
    @IBOutlet var disk1Box: NSBox!
    @IBOutlet var disk2Box: NSBox!

	// Member variables
	weak var m_outgoingRecordControlDelegate: OutgoingRecordControlFromUIDelegate?
    private var m_activeDisks: [Bool] = [false, false]

	//==================================================
	//	UIViewController methods
	//==================================================
    override func viewDidLoad() {
		// Register as a IncomingRecordControlToUIDelegate with the CameraControlInterface,
		// and assign CameraControlInterface as our OutgoingRecordControlFromUIDelegate.
        let appDelegate = NSApplication.shared().delegate as! AppDelegate
        let cameraControlInterface: CameraControlInterface = appDelegate.cameraControlInterface
        m_outgoingRecordControlDelegate = cameraControlInterface
        cameraControlInterface.m_recordControlToUIDelegate = self
        cameraControlInterface.onTransportViewLoaded()

        super.viewDidLoad()
    }

	//==================================================
	//	IBActions
	//==================================================
    @IBAction func recordButtonTapped(_: NSButton) {
        m_outgoingRecordControlDelegate?.onRecordPressed()
    }

    @IBAction func disk1ButtonTapped(_: NSButton) {
        m_outgoingRecordControlDelegate?.onDisk1Pressed()
    }

    @IBAction func disk2ButtonTapped(_: NSButton) {
        m_outgoingRecordControlDelegate?.onDisk2Pressed()
    }

	//==================================================
    //	IncomingRecordControlToUserDelegate methods
	//==================================================
    func setRecording(_ isRecording: Bool) {
        let colorFunc = isRecording ? showRecordingColors : showPreviewColors
        colorFunc()
    }

    func onActiveDisksReceived(_ disk1Active: Bool, _ disk2Active: Bool) {
        m_activeDisks[0] = disk1Active
        m_activeDisks[1] = disk2Active

        let activeString = " \(String.Localized("Transport.Active"))"
        disk1Box.title = String.Localized("Transport.Disk1") + (disk1Active ? activeString : "")
        disk2Box.title = String.Localized("Transport.Disk2") + (disk2Active ? activeString : "")
    }

    func onTimecodeReceived(_ timecode: String) {
        timecodeLabel.stringValue = timecode
    }

    func onRecordTimeRemainingReceived(_ remainingTimes: [String]) {
        disk1RemainingTimeLabel.stringValue = remainingTimes[0]
        disk2RemainingTimeLabel.stringValue = remainingTimes[1]
    }

    func setRecordingError(_ hasRecordingError: Bool) {
        recordingErrorLabel.isHidden = !hasRecordingError
    }
    
    func setRecordTimeWarning(_ recordTimeWarning: RecordTimeWarning, _ cardIndex: Int) {}
	func onTimelapseReceived(_ isTimelapseActive: Bool) {}
	func showPlaybackView(for clipName: String, at playbackSpeed: Int) {}
	func hidePlaybackView() {}
	func updateClipName(_ clipName: String) {}

	//==================================================
	//	UI Control
	//==================================================
    func showRecordingColors() {
        let style = NSMutableParagraphStyle()
        style.alignment = .center
        let font = NSFont.systemFont(ofSize: 66)
        recordButton?.attributedTitle = NSAttributedString(string: "●", attributes: [NSForegroundColorAttributeName: NSColor.red, NSParagraphStyleAttributeName: style, NSFontAttributeName: font])
    }

    func showPreviewColors() {
        let style = NSMutableParagraphStyle()
        style.alignment = .center
        let font = NSFont.systemFont(ofSize: 66)
        recordButton?.attributedTitle = NSAttributedString(string: "●", attributes: [NSForegroundColorAttributeName: NSColor.black, NSParagraphStyleAttributeName: style, NSFontAttributeName: font])
    }
}
