/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

public struct TransportInfo {
	public init() {}
	
    public typealias MediaTransportFlag = CCUPacketTypes.MediaTransportFlag

    public var transportMode: CCUPacketTypes.MediaTransportMode = CCUPacketTypes.MediaTransportMode.Preview
    public var storageMediumDisk1: CCUPacketTypes.ActiveStorageMedium = CCUPacketTypes.ActiveStorageMedium.CFastCard
    public var storageMediumDisk2: CCUPacketTypes.ActiveStorageMedium = CCUPacketTypes.ActiveStorageMedium.CFastCard
    public var speed: Int8 = 0
    public var loop: Bool = false
    public var playAll: Bool = false
    public var disk1Active: Bool = false
    public var disk2Active: Bool = false
    public var timelapseRecording: Bool = false

    func serialize() -> [UInt8] {
        var flags: UInt8 = 0
        if loop { flags = flags | MediaTransportFlag.Loop.rawValue }
        if playAll { flags = flags | MediaTransportFlag.PlayAll.rawValue }
        if disk1Active { flags = flags | MediaTransportFlag.Disk1Active.rawValue }
        if disk2Active { flags = flags | MediaTransportFlag.Disk2Active.rawValue }
        if timelapseRecording { flags = flags | MediaTransportFlag.TimelapseRecording.rawValue }

        var data = [UInt8](repeating: 0, count: 5)
        data[0] = transportMode.rawValue
        data[1] = UInt8(bitPattern: speed)
        data[2] = flags
        data[3] = storageMediumDisk1.rawValue
		data[4] = storageMediumDisk2.rawValue

        return data
    }
}
